/*******************************************************************************
 *
 * Copyright (c) 2007, Dave Whitla
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  * Neither the name of the copyright holder nor the names of contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

package au.net.ocean.maven.plugin.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Created by Dave Whitla on 26/03/2008 at 14:55:42
 *
 * @author <a href="mailto:dave.whitla@ocean.net.au">Dave Whitla</a>
 * @version $Id: Mojo.java 0 26/03/2008 14:55:42 dwhitla $
 */
@Documented
@Target(ElementType.TYPE)
@Retention(RetentionPolicy.RUNTIME)
public @interface Mojo {
    
    /**
     * \@aggregator
     * Flags this Mojo to run it in a multi module way, i.e. aggregate the build with the set of projects listed as modules.
     */
    boolean aggregator() default false;

    /**
     * \@configurator <roleHint>
     * The configurator type to use when injecting parameter values into this Mojo. The value is normally deduced from the Mojo's
     * implementation language, but can be specified to allow a custom ComponentConfigurator implementation to be used.
     * NOTE: This will only be used in very special cases, using a highly controlled vocabulary of possible values.
     * (Elements like this are why it's a good idea to use the descriptor tools.)
     */
    String configurator() default "";

    /**
     * The description of this Mojo's functionality.
     * NOTE: While this is not a required part of the Mojo specification, it SHOULD be provided to enable future tool support for
     * browsing, etc. and for clarity.
     */
    String description() default "";

    /**
     * \@execute phase="<phaseName>" lifecycle="<lifecycleId>" or
     * \@execute phase="<phaseName>" or
     * \@execute goal="<goalName>"
     * When this goal is invoked, it will first invoke a parallel lifecycle, ending at the given phase. If a goal is provided
     * instead of a phase, that goal will be executed in isolation. The execution of either will not affect the current project,
     * but instead make available the ${executedProject} expression if required. An alternate lifecycle can also be provided: for
     * more information see the documentation on the build lifecycle .
     */
    Execution execute() default @Execution();

    /**
     * \@executionStrategy <strategy>
     * Specify the execution strategy. NOTE: Currently supports <code>once-per-session</code>, <code>always</code>.
     */
    ExecutionStrategy executionStrategy() default ExecutionStrategy.OncePerSession;

    /**
     * \@goal <goalName>
     * The name for the Mojo that users will reference from the command line to execute the Mojo directly, or inside a POM in
     * order to provide Mojo-specific configuration.
     */
    String goal();

    /**
     * \@inheritByDefault <true|false>
     * Specify that the Mojo is inherited.
     */
    boolean inheritByDefault() default true;

    /**
     * \@instantiationStrategy <per-lookup>
     * Specify the instantiation strategy.
     */
    InstantiationStrategy instantiationStrategy() default InstantiationStrategy.PerLookup;

    /**
     * \@phase <phaseName>
     * Binds this Mojo to a particular phase of the standard build lifecycle, if specified.
     * NOTE: This is only required if this Mojo is to participate in the standard build process.
     */
    Phase phase() default Phase.None;

    /**
     * \@requiresDependencyResolution <requiredScope>
     * Flags this Mojo as requiring the dependencies in the specified scope (or an implied scope) to be resolved before it can execute.
     * NOTE: Currently supports compile , runtime , and test scopes.
     */
    Scope requiresDependencyResolution() default Scope.None;

    /**
     * \@requiresDirectInvocation <true|false>
     * Flags this Mojo to be invoke directly.
     */
    boolean requiresDirectInvocation() default false;

    /**
     * \@requiresOnline <true|false>
     * Flags this Mojo to be run in online mode.
     */
    boolean requiresOnline() default true;

    /**
     * \@requiresProject <true|false>
     * Flags this Mojo to run inside of a project.
     */
    boolean requiresProject() default true;

    /**
     * \@requiresReports <true|false>
     * Flags this Mojo to require reports.
     */
    boolean requiresReports() default false;

    /**
     * \@deprecated <deprecated-text>
     * Specify the version when the Mojo was deprecated to the API. Similar to Javadoc deprecated. This will trigger a warning
     * when a user tries to configure a parameter marked as deprecated.
     */
    String deprecated() default "";

    /**
     * \@since <since-text>
     * Specify the version when the Mojo was added to the API. Similar to Javadoc since.
     */
//    String since() default "";
}
