package au.net.ocean.maven.plugin.extractor.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

/**
 * This class-cluster is used to represent a file loaded from the class path, and
 * can either be a regular file or a zip file entry.
 */
public abstract class ClassFile {

    public abstract InputStream getInputStream() throws IOException;
    public abstract boolean exists();
    public abstract boolean isDirectory();
    public abstract long lastModified();
    public abstract String getPath();
    public abstract String getName();
    public abstract long length();

    public abstract String getAbsoluteName();

    public static ClassFile wrap(ZipFile zipFile, ZipEntry zipEntry) {
        return new JarClassFile(zipFile, zipEntry);
    }

    public static ClassFile wrap(File file) {
        return new FileSystemClassFile(file);
    }

    private static class JarClassFile extends ClassFile {

        private ZipFile zipFile;
        private ZipEntry zipEntry;

        public JarClassFile(ZipFile zipFile, ZipEntry zipEntry) {
            this.zipFile = zipFile;
            this.zipEntry = zipEntry;
        }

        public InputStream getInputStream() throws IOException {
            try {
                return zipFile.getInputStream(zipEntry);
            } catch (ZipException e) {
                throw new IOException(e.getMessage());
            }
        }

        public boolean exists() {
            return true;
        }

        public boolean isDirectory() {
            return zipEntry.getName().endsWith("/");
        }

        public long lastModified() {
            return zipEntry.getTime();
        }

        public String getPath() {
            return zipFile.getName() + "(" + zipEntry.getName() + ")";
        }

        public String getName() {
            return zipEntry.getName();
        }

        public long length() {
            return zipEntry.getSize();
        }

        public String toString() {
            return zipEntry.toString();
        }

        public String getAbsoluteName() {
            return zipFile.getName() + "(" + zipEntry.getName() + ")";
        }
    }


    private static class FileSystemClassFile extends ClassFile {

        private File file;

        public FileSystemClassFile(File file) {
            this.file = file;
        }

        public InputStream getInputStream() throws IOException {
            return new FileInputStream(file);
        }

        public boolean exists() {
            return file == null || file.exists();
        }

        public boolean isDirectory() {
            return file.isDirectory();
        }

        public long lastModified() {
            return file.lastModified();
        }

        public String getPath() {
            return file.getPath();
        }

        public String getName() {
            return file.getName();
        }

        public long length() {
            return file.length();
        }

        public String toString() {
            return file.toString();
        }

        public String getAbsoluteName() {
            try {
                return file.getCanonicalPath();
            } catch (IOException e) {
                return file.getAbsolutePath();
            }
        }
    }

}
